unit MainWindow;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: Basics                                    //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          //
// -render basic objects,                                   //
// -use various transformations,                            //
// -assign materials and textures to an object.             //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VisClass, VisCam, VisView, VisTex, VisLight, VisMat, VisShape, ExtCtrls,
  Menus,VisAbout,VisDefs, visobj, visattr, VisTimer, VisComp,
  VisPrimitiveObj, VisCanvasView;

type
  TMainWnd = class(TForm)
    View: TVisView;
    Sphere: TVisSphere;
    Cone: TVisCylinder;
    Cube: TVisCylinder;
    SphereMat: TVisMaterial;
    ConeMat: TVisMaterial;
    CubeMat: TVisMaterial;
    Light: TVisLight;
    MarbleTex: TVisTexture;
    MainMenu: TMainMenu;
    File1: TMenuItem;
    Options1: TMenuItem;
    Help1: TMenuItem;
    Exit1: TMenuItem;
    Textures1: TMenuItem;
    About1: TMenuItem;
    cam: TVisPerspectiveCamera;
    attr: TVisAttributes;
    VisTimer1: TVisTimer;
    procedure Exit1Click(Sender: TObject);
    procedure Textures1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ViewRender(aView: TVisView);
    procedure VisTimer1Timer(sender: TObject; lagcount: Integer);
  private
    Step : Double;
  end;

var
  MainWnd: TMainWnd;

implementation

{$R *.DFM}

procedure TMainWnd.Exit1Click(Sender: TObject);
begin
 Close;
end;

procedure TMainWnd.Textures1Click(Sender: TObject);
begin
 //switch texture mapping on/off
 MarbleTex.Enabled:=not(Textures1.Checked);
 Textures1.Checked:=MarbleTex.Enabled;
end;

procedure TMainWnd.About1Click(Sender: TObject);
begin
 ShowAboutDlg;
end;

procedure TMainWnd.ViewRender(aView: TVisView);
begin
 //render all objects:
 Light.Render(aView);
 Sphere.Render(aView);
 Cone.Render(aView);
 Cube.Render(aView);
end;

procedure TMainWnd.VisTimer1Timer(sender: TObject; lagcount: Integer);
begin
 //perform transformations:

 Cone.Transformation.Rotation.incZ(5);
 Cube.Transformation.Rotation.incY(-3);

 //simulate gravity force:
 step:=step-0.001;
 if (Sphere.Transformation.Translation.Y<=0.5) then
  begin
   Step:=0.04;
   Sphere.Transformation.Translation.Y:=0.5;
  end;

 Sphere.Transformation.Translation.incY(step);
 Sphere.Transformation.Rotation.incY(1);

 //redraw the scene
 View.InvalidateGL;
end;

end.
