unit Main;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: Bouncing ball                             //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          //
// -implement drag and drop for three dimensional objects   //
//  using the methods GetObj and WinTo3D,                   //
// -simulate simple gravity and collision testing,          //
// -use moving light sources for special effects.           //
//Click on the ball to drag it. Move the ball before        //
//dropping it to accelerate it.                             //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Graphics, Forms, Menus, StdCtrls, ExtCtrls, Controls, Classes, SysUtils,
  VisMat, VisTimer, VisAttr, VisShape, VisGeom, VisObj, VisPrimitiveObj,
  VisLight, VisComp, VisCam, VisClass, VisCanvasView, VisView, VisMath, VisAbout;

type
  TForm1 = class(TForm)
    VisView1: TVisView;
    VisPerspectiveCamera1: TVisPerspectiveCamera;
    VisNode1: TVisNode;
    VisSphere1: TVisSphere;
    VisBox1: TVisBox;
    VisLight1: TVisLight;
    VisAttributes1: TVisAttributes;
    VisAttributes2: TVisAttributes;
    VisTimer1: TVisTimer;
    VisMaterial1: TVisMaterial;
    VisMaterial3: TVisMaterial;
    VisLight2: TVisLight;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Help1: TMenuItem;
    Exit1: TMenuItem;
    About1: TMenuItem;
    Options1: TMenuItem;
    Transparency1: TMenuItem;
    N1: TMenuItem;
    Highdetail1: TMenuItem;
    Lowdetail1: TMenuItem;
    VisRectangle1: TVisRectangle;
    VisMaterial2: TVisMaterial;
    Showgrid1: TMenuItem;
    procedure VisView1Render(aView: TVisView);
    procedure VisTimer1Timer(sender: TObject; lagcount: Integer);
    procedure VisSphere1AfterInit(Sender: TObject);
    procedure VisView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VisView1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VisView1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Transparency1Click(Sender: TObject);
    procedure Highdetail1Click(Sender: TObject);
    procedure Lowdetail1Click(Sender: TObject);
    procedure Showgrid1Click(Sender: TObject);
  end;

var
  Form1: TForm1;
  velocity: TVisMathVector;
  DoDrag: boolean;
  DragDist: TVisMathVector;
  MousePos, OldPos: TPoint;

implementation

{$R *.DFM}

procedure TForm1.VisView1Render(aView: TVisView);
begin
 if DoDrag then //if sphere is dragged then
  with VisSphere1.Transformation.Translation do
  begin
   //calculate the new position of the sphere:
   //1-((Z+2)/6) calculates the position between the clip planes in this special scene
   AsMathVector:=VisView1.WinTo3D(MousePos.X,MousePos.Y,1-((Z+2)/6));

   //keep the sphere inside the box:
   if X>1-VisSphere1.Radius then X:=1-VisSphere1.Radius;
   if X<VisSphere1.Radius-1 then X:=VisSphere1.Radius-1;
   if Y>1-VisSphere1.Radius then Y:=1-VisSphere1.Radius;
   if Y<VisSphere1.Radius-1 then Y:=VisSphere1.Radius-1;
  end;
 //move the second light with the sphere:
 VisLight1.Transformation.Translation.AsMathVector:=VisSphere1.Transformation.Translation.AsMathVector;

 //Darken back wall:
 //The light is calculated by OpenGL based on its direction
 //and not on its distance. To achieve more realism the back
 //wall is darkened when the light is in the front.
 if VisSphere1.Transformation.Translation.Z>0 then
  VisMaterial2.ColorDiffuse.Color:=$00010101*round((2-VisSphere1.Radius-VisSphere1.Transformation.Translation.Z)/(2-VisSphere1.Radius)*255)
 else VisMaterial2.ColorDiffuse.Color:=clWhite;

 //render all objects:
 VisNode1.Render(aView);
end;

procedure TForm1.VisTimer1Timer(sender: TObject; lagcount: Integer);
begin
if not DoDrag then //move sphere if not dragged:
 with VisSphere1.Transformation.Translation do
  begin
   //move sphere:
   IncX(velocity.x);
   IncY(velocity.y);
   IncZ(velocity.z);
   //check if walls have been hit:
   if abs(X)>1-VisSphere1.Radius then
    begin
     if X>0 then X:=2-2*VisSphere1.Radius-X
      else X:=-2+2*VisSphere1.Radius-X;
     velocity.X:=-velocity.X*0.9; //reverse moving direction and slow the sphere down
    end;
  if abs(Y)>1-VisSphere1.Radius then
    begin
     if Y>0 then Y:=2-2*VisSphere1.Radius-Y
      else Y:=-2+2*VisSphere1.Radius-Y;
     velocity.Y:=-velocity.Y*0.9; //reverse moving direction and slow the sphere down
     if (velocity.Y<0.02) and (Y<VisSphere1.Radius-0.9) then //if sphere is on the ground then...
      begin
       velocity.Y:=0;  //...stop bouncing.
       Y:=VisSphere1.Radius-1;
      end;
    end;
  if abs(Z)>2-VisSphere1.Radius then
    begin
     if Z>0 then Z:=4-2*VisSphere1.Radius-Z
      else Z:=-4+2*VisSphere1.Radius-Z;
     velocity.Z:=-velocity.Z*0.9; //reverse moving direction and slow the sphere down
    end;
   if (velocity.y>0) or (y>VisSphere1.Radius-1) then //add gravity force
    velocity.y:=velocity.y-0.01;
  end;
 VisView1.InvalidateGL; //redraw the scene
end;

procedure TForm1.VisSphere1AfterInit(Sender: TObject);
begin
 velocity:=ToVec(-0.05, 0.025, -0.04); //init velocity
end;

procedure TForm1.VisView1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var obj: TObject;
begin
 VisView1.GetObj(X,Y,obj); //check if sphere was clicked
 if assigned(obj) and (obj is TVisSphere) then
  begin
   DoDrag:=true; //start dragging the sphere
   MousePos:=Point(X,Y);
   OldPos:=Point(X,Y);
   velocity.z:=-0.05;
  end;
end;

procedure TForm1.VisView1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 if DoDrag then //drop the sphere:
  begin
   DoDrag:=false;
   //calculate velocity from old and current mouse position:
   velocity.x:=(MousePos.X-OldPos.X)/VisView1.Width;
   velocity.y:=(OldPos.Y-MousePos.Y)/VisView1.Height;
  end;
end;

procedure TForm1.VisView1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
 //drag the sphere:
 if DoDrag then
  begin
   OldPos:=MousePos; //update mouse position
   MousePos:=Point(X,Y);
  end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
 Close;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
 ShowAboutDlg;
end;

procedure TForm1.Transparency1Click(Sender: TObject);
begin
 Transparency1.Checked:=not Transparency1.Checked;
 if Transparency1.Checked then
  VisAttributes2.Transparency:=tmAddAlpha
 else VisAttributes2.Transparency:=tmNone;
end;

procedure TForm1.Highdetail1Click(Sender: TObject);
begin
 HighDetail1.Checked:=true;
 VisSphere1.Slices:=32;
 VisBox1.DetailX:=20;
 VisBox1.DetailY:=10;
 VisBox1.DetailZ:=20;
 VisRectangle1.DetailX:=10;
 VisRectangle1.DetailY:=10;
end;

procedure TForm1.Lowdetail1Click(Sender: TObject);
begin
 Lowdetail1.Checked:=true;
 VisSphere1.Slices:=16;
 VisBox1.DetailX:=5;
 VisBox1.DetailY:=5;
 VisBox1.DetailZ:=5;
 VisRectangle1.DetailX:=5;
 VisRectangle1.DetailY:=5;
end;

procedure TForm1.Showgrid1Click(Sender: TObject);
begin
 Showgrid1.Checked:=not Showgrid1.Checked;
 if Showgrid1.Checked then
  VisAttributes1.PolygonMode:=pmLine
 else VisAttributes1.PolygonMode:=pmFill;
end;

end.
