unit Unit1;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: Extrusion                                 //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          //
// -extrude different types of objects,                     //
// -use simple transparency effects,                        //
// -initialize a TVisPolygon,                               //
// -delete points from an object.                           //
//Click on the button to proceed step by step.              //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, VisClass, VisCanvasView, VisView, ExtCtrls, VisTimer, VisCam,
  Menus, VisObj, VisPrimitiveObj, VisPoly, VisMat, VisLight, VisAttr,
  VisShape, VisAbout, VisComp, VisGL;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    VisPerspectiveCamera1: TVisPerspectiveCamera;
    VisTimer1: TVisTimer;
    Panel1: TPanel;
    VisView1: TVisView;
    Panel2: TPanel;
    Button1: TButton;
    Edit1: TEdit;
    VisNode1: TVisNode;
    VisAttributes1: TVisAttributes;
    VisLight1: TVisLight;
    VisMaterial1: TVisMaterial;
    Polygon: TVisPolygon;
    CheckBox1: TCheckBox;
    Sphere: TVisSphere;
    VisAttributes2: TVisAttributes;
    VisMaterial2: TVisMaterial;
    File1: TMenuItem;
    Help1: TMenuItem;
    Exit1: TMenuItem;
    About1: TMenuItem;
    Edit2: TEdit;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure VisView1Render(aView: TVisView);
    procedure Button1Click(Sender: TObject);
    procedure VisTimer1Timer(sender: TObject; lagcount: Integer);
    procedure CheckBox1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure PolygonConstruct(Sender: TObject);
    procedure SphereBeforeRender(aView: TVisCustomView);
    procedure SphereAfterRender(aView: TVisCustomView);
  end;

var
  Form1: TForm1;
  CurStep: integer=0;

implementation

{$R *.DFM}

//Proceed one step of the demo:
procedure DoStep;
begin
Form1.Edit2.Text:=Form1.Edit1.Text;
 case CurStep of
  0: begin //init polygon
      Form1.Polygon.Init;
      Form1.Sphere.Enabled:=false;
      Form1.VisAttributes1.Lighting:=false;
      Form1.Edit1.Text:='Polygon.Extrude(0.2, 3, false);';
     end;
  1: begin
      //extrude the polygon line:
      Form1.Polygon.Extrude(0.15, 3, false);
      //Note: The parameter smooth is false for all extrusions in this demo.
      //Try setting it to true in the above call to see its effects.
      Form1.VisAttributes1.Lighting:=true;
      Form1.Edit1.Text:='Polygon.Extrude(0.15, 3, false);';
     end;
  2: begin
      Form1.Polygon.Extrude(0.15, 3, false);
      Form1.Edit1.Text:='Sphere.Enabled:=true;  Sphere.Init;';
     end;
  3: begin
      Form1.Sphere.Enabled:=true;
      Form1.Sphere.Init;
      Form1.VisAttributes2.CullFace:=cfnone;
      Form1.Edit1.Text:='Sphere.FreePoints(10,150);';
     end;
  4: begin //free the points 10 to 160 (only to cut an arbitrary hole into the sphere)
      Form1.Sphere.FreePoints(10,150);
      Form1.Edit1.Text:='Sphere.Extrude(-0.1, 3, false);';
     end;
  5: begin
      Form1.Sphere.Extrude(-0.1, 3, false);
      Form1.VisAttributes2.CullFace:=cfback;
      Form1.Edit1.Text:='<reset scene>';
     end;
 end; //of case
 CurStep:=(CurStep+1) mod 6;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 DoStep;
end;

procedure TForm1.VisView1Render(aView: TVisView);
begin
 VisNode1.Render(aView);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
 DoStep;
end;

procedure TForm1.VisTimer1Timer(sender: TObject; lagcount: Integer);
begin
 Polygon.Transformation.Rotation.incX(3);
 Polygon.Transformation.Rotation.incY(4);
 Sphere.Transformation.Rotation.incX(3);
 Sphere.Transformation.Rotation.incZ(1);
 VisView1.InvalidateGL;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
 if CheckBox1.Checked then
 begin
  VisAttributes1.PolygonMode:=pmLine;
  VisAttributes2.PolygonMode:=pmLine;
 end else begin
  VisAttributes1.PolygonMode:=pmFill;
  VisAttributes2.PolygonMode:=pmFill;
 end;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
showAboutDlg;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
 close;
end;

procedure TForm1.PolygonConstruct(Sender: TObject);
begin
 Polygon.AddValues(0  ,0  ,0,0,0);
 Polygon.AddValues(0.5,0  ,0,0,0);
 Polygon.AddValues(0.3,0.3,0,0,0);
 Polygon.AddValues(0  ,0.5,0,0,0);
 Polygon.CalcNormals;
end;

procedure TForm1.SphereBeforeRender(aView: TVisCustomView);
begin
 //The sphere does not draw its depth values so that
 //all parts (primitives) of the sphere are rendered no
 //matter in which order they occur. This is necessary because
 //the sphere is transparent.
 //Try to remove this call to see the effect.
 VisView1.DrawValues:=VisView1.DrawValues-[dvDepth];
end;

procedure TForm1.SphereAfterRender(aView: TVisCustomView);
begin
 //reset depth draw settings:
 VisView1.DrawValues:=VisView1.DrawValues+[dvDepth];
end;

end.

