unit MainWindow;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: MazeWalk                                  //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          //
// -use fog effects to create the impression of darkness,   //
// -set alpha bitmaps for irregular transparent textures,   //
//Use the arrow keys to walk through the maze.              //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VisClass, VisCam, VisView, VisLight, VisTex, VisMat, ExtCtrls,
  VisShape,  VisObj, VisAttr, VisGeom, VisPrimitiveObj, VisCanvasView,
  VisComp, VisAbout, Menus;

type
 TPlayer = record
  XPos,ZPos,Dir : Integer;
 end;

type
  TMainWnd = class(TForm)
    MazeView: TVisView;
    GroundTex: TVisTexture;
    WallTex: TVisTexture;
    mazeCamera: TVisPerspectiveCamera;
    mazeattr: TVisAttributes;
    Box: TVisBox;
    MazeFog: TVisFog;
    MapTex: TVisTexture;
    shadowtex: TVisTexture;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;Shift: TShiftState);
    procedure MazeViewRender(aView: TVisView);
    procedure FormActivate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    Player : TPlayer;
    procedure DrawGround;
    procedure DrawBorders;
    procedure SetPlayerToMaze;
    procedure UpdateViews;
  public
  end;

var
  MainWnd: TMainWnd;

implementation

{$R *.DFM}


const
 widthMaze  = 20;
 heightMaze = 20;
 fac        = 0.5;
 fldWay     = 1;
 fldWall    = 0;
 dirUp      = 0;
 dirRight   = 1;
 dirDown    = 2;
 dirLeft    = 3;

 Maze : array[0..heightMaze+1,0..widthMaze+1] of Byte =
        ((0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
         (0,1,1,0,1,1,0,1,0,1,0,1,1,1,0,1,1,1,1,1,1,0),
         (0,1,0,0,1,0,0,1,0,1,0,1,0,1,0,1,0,0,0,0,0,0),
         (0,1,0,1,1,1,1,1,0,1,0,1,0,1,0,1,0,1,1,1,0,0),
         (0,1,1,1,0,0,0,0,0,1,1,1,0,1,0,1,0,1,0,1,0,0),
         (0,0,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0,1,0,1,0,0),
         (0,1,1,1,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,1,0,0),
         (0,1,0,0,0,1,0,1,1,1,1,1,0,1,1,1,1,1,1,1,1,0),
         (0,1,1,1,1,1,0,0,1,0,1,0,0,1,0,0,0,0,0,1,0,0),
         (0,0,0,0,0,1,1,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0),
         (0,1,1,1,0,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,0),
         (0,1,0,1,0,1,0,1,1,1,1,1,1,1,1,1,0,1,0,0,0,0),
         (0,1,0,1,0,1,0,0,0,1,0,1,0,0,0,1,0,1,1,1,1,0),
         (0,1,0,1,1,1,1,1,1,1,0,1,1,1,0,1,0,0,0,0,1,0),
         (0,1,0,1,0,0,0,0,0,0,0,0,0,1,0,1,1,1,1,1,1,0),
         (0,1,0,1,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,0,1,0),
         (0,1,0,1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,0),
         (0,1,0,1,0,1,0,0,0,0,0,1,0,0,1,0,0,0,1,0,1,0),
         (0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0),
         (0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,1,0),
         (0,1,1,1,1,1,0,1,1,1,1,1,1,1,1,0,1,1,1,1,1,0),
         (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0));

procedure TMainWnd.FormCreate(Sender: TObject);
begin
 //init maze
 Box.Transformation.Scale.X:=fac;
 Box.Transformation.Scale.Y:=fac;
 Box.Transformation.Scale.Z:=fac;
 MazeCamera.Transformation.translation.Y:=-fac / 2;
 with Player do begin
  XPos:=1;
  ZPos:=1;
  Dir:=dirDown;
 end;
 SetPlayerToMaze;
end;

procedure TMainWnd.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

 procedure Go(var zp,xp : Integer;zg,xg : Integer);
 begin
  if (zg<>0)and(Maze[zp+zg,xp]=fldWay) then zp:=zp+zg;
  if (xg<>0)and(Maze[zp,xp+xg]=fldWay) then xp:=xp+xg;
 end;

 procedure Turn(var cDir : Integer;aDir : Integer);
 begin
  cDir:=cDir+aDir;
  if (cDir<dirUp) then cDir:=dirLeft;
  if (cDir>dirLeft) then cDir:=dirUp;
 end;

var
 f : Integer;
begin
 with Player  do begin
  case Key of
   VK_UP,VK_DOWN: // move
   begin
    if (Key=VK_DOWN) then f:=-1 else f:=1;
    case Dir of
     dirUp   : Go(ZPos,XPos,-1*f, 0);
     dirDown : Go(ZPos,XPos, 1*f, 0);
     dirLeft : Go(ZPos,XPos, 0  ,-1*f);
     dirRight: Go(ZPos,XPos, 0  , 1*f);
    end;
   end;
   VK_LEFT,VK_RIGHT: // turn
   begin
    if (Key=VK_LEFT) then Turn(Dir,-1) else Turn(Dir,1);
   end;
  end;
 end;
 UpdateViews;
end;

//Draw maze floor:
procedure TMainWnd.DrawGround;
var
  x,z : Integer;
begin
Box.Texture:=GroundTex;
Box.Faces:=[bfbottom];
for x:=1 to (widthMaze) do
  for z:=1 to (heightMaze) do
    begin
      Box.Transformation.Translation.x:=x+0.5;
      Box.Transformation.Translation.z:=z;
      if (Maze[z,x]=fldWay) then Box.Render(MazeView);
    end;
end;

//Draw maze walls:
procedure TMainWnd.DrawBorders;
var
  x,z : Integer;
begin
box.Faces:=[];
Box.Texture:=WallTex;
for x:=1 to (widthMaze) do
  for z:=1 to (heightMaze) do
    if (Maze[z,x]=fldWay) then
    begin
      box.Faces:=[];
      if (Maze[z-1,x]=fldWall) then box.Faces:=box.Faces+[bfback];
      if (Maze[z,x+1]=fldWall) then box.Faces:=box.Faces+[bfright];
      if (Maze[z+1,x]=fldWall) then box.Faces:=box.Faces+[bffront];
      if (Maze[z,x-1]=fldWall) then box.Faces:=box.Faces+[bfleft];
      Box.Transformation.Translation.x:=x+0.5;
      Box.Transformation.Translation.z:=z;
      Box.Render(MazeView);
    end;
end;

//Position the camera:
procedure TMainWnd.SetPlayerToMaze;
begin
 with Player do begin
  // Players position
  MazeCamera.Transformation.translation.X:=-(fac * XPos) - (fac / 2);
  MazeCamera.Transformation.translation.Z:=-(fac * ZPos) - (fac / 2);
  // Players view direction
  case Dir of
   dirUp    : MazeCamera.Transformation.Rotation.Y:=0;
   dirDown  : MazeCamera.Transformation.Rotation.Y:=180;
   dirLeft  : MazeCamera.Transformation.Rotation.Y:=-90;
   dirRight : MazeCamera.Transformation.Rotation.Y:=90;
  end;//case Dir
 end;//with Player
end;

procedure TMainWnd.UpdateViews;
begin
 SetPlayerToMaze;
 MazeView.InvalidateGL;
end;

//Render the view:
procedure TMainWnd.MazeViewRender(aView: TVisView);
begin
  MazeFog.Render(aView);
  mazeattr.render(aview);
  GroundTex.render(aview);
  DrawGround;
  WallTex.render(aview);
  DrawBorders;
  
  //draw the map:
  MazeView.Canvas.Transparency:=tmAlpha;
  MazeView.Canvas.DrawTexture(bounds(20,20,128,128),MapTex,rect(round(Player.XPos/widthMaze*256)-64,round(Player.ZPos/heightMaze*256)-64,round(Player.XPos/widthMaze*256)+64,round(Player.ZPos/heightMaze*256)+64));
  MazeView.Canvas.Brush.Color:=cldkgray;
  MazeView.Canvas.DrawTexture(bounds(20,20,128,128),shadowtex,bounds(0,0,128,128));
  MazeView.Canvas.FrameRect(bounds(20,20,128,128));
end;

//init the bitmaps and textures:
procedure TMainWnd.FormActivate(Sender: TObject);
var bmp:TBitMap;
     a,b:integer;
begin
 MazeView.Current:=true;

 bmp:=TBitmap.Create;
 bmp.width:=128; bmp.height:=128;
 bmp.canvas.brush.color:=clblack;
 bmp.canvas.FillRect(rect(0,0,128,128));
 shadowtex.bitmap:=bmp;
 shadowtex.SetAlphaBitmap(MapTex.bitmap);

 bmp.width:=220;   bmp.height:=220;
 for a:=0 to heightMaze+1 do
  for b:=0 to widthmaze+1 do
   begin
    if maze[a,b]=fldWall then
     bmp.canvas.brush.color:=clblack
    else bmp.canvas.brush.color:=clwhite;
    bmp.Canvas.FillRect(rect(b*10,a*10,(b+1)*10,(a+1)*10));
   end;
 Maptex.Bitmap:=bmp;
end;

procedure TMainWnd.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMainWnd.About1Click(Sender: TObject);
begin
  ShowAboutDlg;
end;

end.
