unit Main;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: Robot vehicle                             //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          // 
// -perform measurements in an artificial environment       //
//  using the method GetPoint,                              //
// -create advanced objects using SetReliefBitmap,          //
// -use texture mapping, fog, light, materials and nodes,   //
// -apply basic VisCanvas methods,                          //  
// -speed up the rendering by lowering the detail and by    //
//  switching off unimportant features.                     //
//The robot within the scene is controlled based only on    //
//distance information from three scan points. The scene is //
//created randomly.                                         //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VisCntrl, VisMat, VisShape, VisObj, VisPrimitiveObj, VisGeom, VisLight,
  VisAttr, VisComp, VisCam, VisClass, VisCanvasView, VisView, VisMath,
  VisTimer, VisAbout, Menus, VisTex;

type
  TForm1 = class(TForm)
    VisView1: TVisView;
    OverviewCam: TVisPerspectiveCamera;
    SceneAttr: TVisAttributes;
    GroundRect: TVisRectangle;
    Box: TVisBox;
    VisLight1: TVisLight;
    GroundMat: TVisMaterial;
    VisMouseController1: TVisMouseController;
    BoxMat: TVisMaterial;
    RoboNode: TVisNode;
    VisBox2: TVisBox;
    RoboMat: TVisMaterial;
    VisTimer1: TVisTimer;
    RoboCam: TVisPerspectiveCamera;
    BoundBox: TVisBox;
    BoundAttr: TVisAttributes;
    RoboAttr: TVisAttributes;
    Wheel1: TVisCylinder;
    Wheel2: TVisCylinder;
    WheelMat: TVisMaterial;
    Wheel3: TVisCylinder;
    Wheel4: TVisCylinder;
    RoboSphere: TVisSphere;
    RoboMat2: TVisMaterial;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Options1: TMenuItem;
    RoboCam1: TMenuItem;
    OverviewCam1: TMenuItem;
    N1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    AboutDemo1: TMenuItem;
    VisTexture1: TVisTexture;
    VisTexture2: TVisTexture;
    FastBox: TVisBox;
    FastAttr: TVisAttributes;
    VisFog1: TVisFog;
    Fulldetail1: TMenuItem;
    NoDetail1: TMenuItem;
    New1: TMenuItem;
    procedure VisView1Render(aView: TVisView);
    procedure FormCreate(Sender: TObject);
    procedure VisTimer1Timer(sender: TObject; lagcount: Integer);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure OverviewCam1Click(Sender: TObject);
    procedure RoboCam1Click(Sender: TObject);
    procedure GroundRectAfterInit(Sender: TObject);
    procedure BoxAfterInit(Sender: TObject);
    procedure NoDetail1Click(Sender: TObject);
    procedure Fulldetail1Click(Sender: TObject);
    procedure AboutDemo1Click(Sender: TObject);
    procedure FastBoxAfterInit(Sender: TObject);
    procedure New1Click(Sender: TObject);
  end;

type TBoxData= record
                transform, dims: TVisMathVector;
               end;

var
  Form1: TForm1;
  Boxes: array[0..9] of TBoxData;  //obstacles
  TurnAngle: integer;  //angle the robot still has to be turned
  OVCam: boolean;  //OvervieCam used?
  SensorData: array [0..2] of double;  //data from the scan points
  speed: double=0; //current speed of the robot

implementation

uses AboutDemo;

{$R *.DFM}

procedure CreateEnvironment;
var a:integer;
begin
 for a:=0 to 9 do
  begin
   Boxes[a].Transform:=ToVec(random(90)/100+0.1,random(90)/100+0.1,random(360));
   Boxes[a].Dims:=ToVec(random(8)/100+0.02,random(10)/100+0.06,random(8)/100+0.02);
  end;
end;

procedure TForm1.VisView1Render(aView: TVisView);
var a,r:integer;
    CurBox: TVisBox;
    p: TVisMathVector;
begin
 //The scene is rendered mutiple times when performing GetPoint
 //To speed this up the fog, light, ground and textures are not
 //used in this case (DrawMode<>dmWindow).

 if aView.DrawMode=dmWindow then
  begin
   CurBox:=Box; //full effects
   VisFog1.Render(aView);
   VisLight1.Render(aView);
   GroundRect.Render(aView);
  end else CurBox:=FastBox; //no effects

 for a:=0 to 9 do //render obstacles
  begin
   CurBox.Transformation.Translation.AsMathVector:=ToVec(Boxes[a].Transform.X,0,Boxes[a].Transform.Y);
   CurBox.Transformation.Rotation.Y:=Boxes[a].Transform.Z;
   CurBox.Width:=Boxes[a].Dims.X;
   CurBox.Height:=Boxes[a].Dims.Y;
   CurBox.Depth:=Boxes[a].Dims.Z;
   CurBox.Render(aView);
  end;
 //BoundBox is used as an outer wall during scanning
 if (aView.DrawMode<>dmWindow) or not OVCam then
  BoundBox.Render(aView);

 //The scanning is perfomed during rendering if possible to improve the
 //performance. This is only possible if the RoboCam is active.
 if not OVCam then
   for a:=0 to 2 do
    begin //scan
     VisView1.GetPoint(round((VisView1.Width/4)*(1+a)),VisView1.Height div 2,p);
     SensorData[a]:=sqrt(sqr(RoboNode.Transformation.Translation.Z-p.z)+sqr(RoboNode.Transformation.Translation.X-p.x));
    end;

 if aView.DrawMode=dmWindow then
  begin
   RoboNode.Render(aView);  //draw robot
   if aView.Camera.Name='RoboCam' then  //draw robot display
    with aView.Canvas do
     begin
      for a:=1 to 3 do //draw scan points:
       begin
        if SensorData[a-1]<0.15 then
         Pen.Color:=clRed else Pen.Color:=$00111111;
        MoveTo(round(aView.Width/4*a)-5,aView.Height div 2);
        LineTo(round(aView.Width/4*a)+5,aView.Height div 2);
        MoveTo(round(aView.Width/4*a),aView.Height div 2-5);
        LineTo(round(aView.Width/4*a),aView.Height div 2+5);
        TextOut(round(aView.Width/4*a)-13,aView.Height div 2+16,FloatToStrF(SensorData[a-1],ffFixed,2,3));
       end;
      //show direction:
      Brush.Alpha:=0.3; Brush.Color:=clWhite;
      Pen.Color:=clBlack;
      r:=aView.Width div 13;
      Ellipse(aView.Width-2*r-5, 5, aView.Width-5, 2*r+5);

      MoveTo(aView.Width-5-r, 5+r);
      Pen.Width:=3;  Pen.Color:=clRed; Pen.Alpha:=0.5; // r:=round(r*0.9);
      LineTo(round(aView.Width-5-r-r*sin(RoboNode.Transformation.Rotation.Y/180*pi)),
             round(5+r-r*cos(RoboNode.Transformation.Rotation.Y/180*pi)));
      //draw rectangles:
      Pen.Width:=1; Pen.Alpha:=0.8;
      Brush.Color:=clWhite;
      Pen.Color:=clBlack;
      Rectangle(rect(3,4,146,27));
      Rectangle(rect(146,4,171,27));
      Rectangle(rect(aView.Width-2*r-50,4,aView.Width-2*r-10,27));
      Rectangle(rect(aView.Width div 2,4,aView.Width div 2+80,27));
      Rectangle(rect(aView.Width div 2-50,aView.Height-37,aView.Width div 2+50,aView.Height-14));
      Brush.Color:=clRed;  Brush.Alpha:=0.5;
      //speed bar
      FillRect(rect(5,5,5+round(speed*20000),25));

      //draw text:
      TextOut(151,20,FloatToStrF(round(Speed*10000)/10,ffGeneral,1,2));
      TextOut(aView.Width-2*r-40,20,IntToStr(round(RoboNode.Transformation.Rotation.Y))+'');
      TextOut(aView.Width div 2-45,aView.Height-21,'FRAMERATE '+inttostr(VisTimer1.FrameRate));
      if (TurnAngle>=0) and (Speed=0) then
       TextOut(aView.Width div 2+20,20,'TURN')
      else if (TurnAngle>=0) and (Speed<>0) then
        TextOut(aView.Width div 2+17,20,'BRAKE')
       else if TurnAngle=-1 then
         TextOut(aView.Width div 2+5,20,'ACCELERATE')
     end;
  end;
end; {TForm1.Render}

procedure TForm1.FormCreate(Sender: TObject);
begin
 //init boxes
 Randomize;
 CreateEnvironment;

 //init cam-pos
 RoboCam.Transformation.Rotation.Y:=-RoboNode.Transformation.Rotation.Y;
 RoboCam.Transformation.Translation.X:=-RoboNode.Transformation.Translation.X;
 RoboCam.Transformation.Translation.Z:=-RoboNode.Transformation.Translation.Z;

 OVCam:=false;

 with VisView1.Canvas do
  begin
   Transparency:=tmAlpha;
   Pen.Alpha:=0.8;
   Font.Alpha:=0.8;
  end;
end;

procedure TForm1.VisTimer1Timer(sender: TObject; lagcount: Integer);
var p: TVisMathVector;
    m: double;
    a: integer;
begin
//The scanning is allways done (offscreen) from the robots
//position:
VisView1.Camera:=RoboCam;

//scan environment:
if TurnAngle=-1 then
 begin
  //The scanning is performed during rendering if the RoboCam is active.
  //In overview mode the scanning is done here:
  if OVCam then
   for a:=0 to 2 do
    begin //scan
     VisView1.GetPoint(round((VisView1.Width/4)*(1+a)),VisView1.Height div 2,p);
     SensorData[a]:=sqrt(sqr(RoboNode.Transformation.Translation.Z-p.z)+sqr(RoboNode.Transformation.Translation.X-p.x));
    end;
   m:=SensorData[0];
   for a:=1 to 2 do //find least distance
    if SensorData[a]<m then m:=SensorData[a];

   //if the distance of the closest scanpoint is less then
   //0.15 then the robot turns to the left by 10 degrees:
   if m<0.15 then TurnAngle:=10 //initialize turning
    else if speed<0.007 then speed:=speed+0.0006 //accelerate
     else speed:=speed+0.0005;
 end;

 //turn vehicle
 if TurnAngle>0 then
  begin
   RoboNode.Transformation.Rotation.IncY(1);
   Wheel1.Transformation.Rotation.incZ(2);
   Wheel2.Transformation.Rotation.incZ(2);
   Wheel3.Transformation.Rotation.incZ(-2);
   Wheel4.Transformation.Rotation.incZ(-2);
  end;
 if TurnAngle>=0 then dec(TurnAngle);

 //move vehicle
 if speed>0 then
  begin
   speed:=speed-0.0005;  //brake
   if speed<0 then speed:=0;
   RoboNode.Transformation.Translation.IncX(-speed*sin(RoboNode.Transformation.Rotation.Y/180*pi));
   RoboNode.Transformation.Translation.IncZ(-speed*cos(RoboNode.Transformation.Rotation.Y/180*pi));
   m:=-speed/0.13*360;
   Wheel1.Transformation.Rotation.incZ(m);
   Wheel2.Transformation.Rotation.incZ(m);
   Wheel3.Transformation.Rotation.incZ(m);
   Wheel4.Transformation.Rotation.incZ(m);
  end;

 if NoDetail1.Checked then
  begin //flat ground
   RoboNode.Transformation.Rotation.X:=0;
   RoboNode.Transformation.Rotation.Z:=0;
  end else begin //simulate bumbs
   RoboNode.Transformation.Rotation.X:=sin(RoboNode.Transformation.Rotation.Y/10+(RoboNode.Transformation.Translation.X+RoboNode.Transformation.Translation.Z)*100)*1;
   RoboNode.Transformation.Rotation.Z:=sin(RoboNode.Transformation.Rotation.Y/10+(RoboNode.Transformation.Translation.X+RoboNode.Transformation.Translation.Z)*150)*1;
  end;

 //move camera
 RoboCam.Transformation.Rotation.X:=-RoboNode.Transformation.Rotation.X;
 RoboCam.Transformation.Rotation.Y:=-RoboNode.Transformation.Rotation.Y;
 RoboCam.Transformation.Rotation.Z:=-RoboNode.Transformation.Rotation.Z;
 RoboCam.Transformation.Translation.X:=-RoboNode.Transformation.Translation.X;
 RoboCam.Transformation.Translation.Z:=-RoboNode.Transformation.Translation.Z;

 //reset camera if overview is chosen:
 if OVCam then VisView1.Camera:=OverviewCam;

 VisView1.InvalidateGL; //redraw the scene
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
 Close;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
 ShowAboutDlg;
end;

procedure TForm1.OverviewCam1Click(Sender: TObject);
begin
 //change camera to overview:
 OVCam:=true;
 OverviewCam1.Checked:=true;
 VisFog1.EndDepth:=2;
 RoboSphere.Enabled:=true;
 Wheel1.Slices:=8;
 Wheel3.Slices:=8;
 Wheel2.Enabled:=true;
 Wheel4.Enabled:=true;
 VisView1.Controller:=VisMouseController1;
end;

procedure TForm1.RoboCam1Click(Sender: TObject);
begin
 //change camera to RoboCam
 OVCam:=false;
 RoboCam1.Checked:=true;
 VisFog1.EndDepth:=0.5;
 RoboSphere.Enabled:=false;
 Wheel1.Slices:=20;
 Wheel3.Slices:=20;
 Wheel2.Enabled:=false;
 Wheel4.Enabled:=false;
 VisView1.Controller:=nil;
end;

procedure TForm1.GroundRectAfterInit(Sender: TObject);
begin
 //The texture of the ground is also used as a relief:
 if Fulldetail1.Checked then
  GroundRect.SetReliefBitmap(VisTexture1.Bitmap,0.015,2,2);
end;

procedure TForm1.BoxAfterInit(Sender: TObject);
begin
 //when the detail is low the relief does not resemble the bitmap
 //but causes interesting distortions as for the obstacles:
 if Fulldetail1.Checked then
  Box.SetReliefBitmap(VisTexture2.Bitmap,0.05,2,2);
end;

procedure TForm1.NoDetail1Click(Sender: TObject);
begin
 //lower detail:
 NoDetail1.Checked:=true;
 with Box do
  begin
   Textured:=false;
   DetailX:=1; DetailY:=1; DetailZ:=1;
   Material:=BoxMat;
  end;
 FastBox.DetailY:=1;
 with GroundRect do
  begin
   Textured:=false;
   DetailX:=1; DetailY:=1;
   Material:=GroundMat;
  end;
 VisFog1.Enabled:=false;
 RoboCam.ViewFar:=2;
 SceneAttr.ColorAmbient.Color:=$00111111;
 SceneAttr.ShadeModel:=smFlat;
end;

procedure TForm1.Fulldetail1Click(Sender: TObject);
begin
 //full detail:
 Fulldetail1.Checked:=true;
 with Box do
  begin
   Textured:=true;
   DetailX:=3; DetailY:=3; DetailZ:=3;
   Material:=nil;
  end;
 //Only the detail of the FastBox is raised. Raising all
 //details would be little more accurate but much slower:
 FastBox.DetailY:=3;
 with GroundRect do
  begin
   Textured:=true;
   //the high ground detail is necessary for the relief:
   DetailX:=50; DetailY:=50;
   Material:=nil;
  end;
 VisFog1.Enabled:=true;
 //ViewFar can be low because the fog covers the scene:
 RoboCam.ViewFar:=0.5;
 SceneAttr.ColorAmbient.Color:=clYellow;
 SceneAttr.ShadeModel:=smSmooth;
end;

procedure TForm1.AboutDemo1Click(Sender: TObject);
begin
 Form2.ShowModal;
end;

procedure TForm1.FastBoxAfterInit(Sender: TObject);
begin
 //distort the FastBox (used only scanning) in the same
 //way like the displayed box:
 if Fulldetail1.Checked then
  FastBox.SetReliefBitmap(VisTexture2.Bitmap,0.05,2,2);
end;

procedure TForm1.New1Click(Sender: TObject);
begin
 CreateEnvironment;
end;

end.
