unit Unit1;

//////////////////////////////////////////////////////////////
//Signsoft VisIt: VisTube                                   //
//////////////////////////////////////////////////////////////
//This program demonstrates how to                          //
// -create various objects using the TVisTube component,    //
// -use the TVisExchange component to store and load VisIt  //
//  components
//Use the menu to switch between the objects and to save    //
//and load components.                                      //
//////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  VisAbout, Menus, visshape, VisCustomFuncObj, VisTube, vistex, vislight,
  visobj, viscntrl, visclass, viscam, visview, VisTimer, VisAttr,
  VisExchange, VisPrimitiveObj, VisComp, VisCanvasView;

type
  TForm1 = class(TForm)
    VisView1: TVisView;
    VisMouseController1: TVisMouseController;
    VisNode1: TVisNode;
    VisLight1: TVisLight;
    VisTube1: TVisTube;
    MainMenu: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Options1: TMenuItem;
    MenuCup: TMenuItem;
    MenuBottle: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    VisTimer1: TVisTimer;
    VisAttributes1: TVisAttributes;
    VisPerspectiveCamera1: TVisPerspectiveCamera;
    Save1: TMenuItem;
    VisExchange1: TVisExchange;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    Load1: TMenuItem;
    Scoop1: TMenuItem;
    Spiral1: TMenuItem;
    Random1: TMenuItem;
    VisNode2: TVisNode;
    procedure VisView1Render(aView: TVisView);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure InitTube(index:integer);
    procedure MenuCupClick(Sender: TObject);
    procedure VisTimer1Timer(sender: TObject; lagcount: Integer);
    procedure Save1Click(Sender: TObject);
    procedure Load1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;
  Index: integer;

implementation

{$R *.DFM}

procedure TForm1.VisView1Render(aView: TVisView);
begin
//render all components
VisNode1.Render(aView);
end;

procedure TForm1.About1Click(Sender: TObject);
begin
ShowAboutDlg;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
Close;
end;

procedure TForm1.InitTube(index:integer);
var a:byte;
begin
//Create a new object with the Tube
with VisTube1 do
 begin
  Clear;
  Slices:=30;
  case index of
   1: begin //Create a Cup
       //start at center of the bottom
       StartSegment.Radius:=0.001;
       {note: In most cases the radius should not be set
        to 0 if a section is visible because the light
        rendering would not work properly}

       //add bottom
       AddLine(0,0,0.05,0.4,2,rcEndSmooth);
       //add outside wall
       AddLine(0,0,0.7,0.4,2);
       //add round upper edge
       AddLine(0,0,0.05,0.355,3,rcStartSmooth);
       AddLine(0,0,-0.05,0.35,3,rcEndSmooth);
       //add inside wall
       AddLine(0,0,-0.65,0.35,3);
       //add inside bottom
       AddLine(0,0,0,0.001,2,rcStartSmooth);
       {note: now we can set the radius to 0 because the
       next section will be almost invisible because of
       its location and its small size}
       //move (invisible) inside the wall (radius=0)
       AddLine(0,0,0,0,1);
       AddLine(0.4,0,0,0,1);
       AddLine(0,0,0.05,0,1).EndAngleX:=270;
       AddLine(0,0,0,0.05,1);
       //appear at the outside wall and add the handle
       AddBend(0.2,0,0.25,0.05).EndAngleY:=90;
       AddBend(-0.21,0,0.25,0.05).EndAngleY:=90;
      end;
   2: begin //Create a simple Bottle
       //start at center of the bottom
       StartSegment.Radius:=0.001;
       //add the bottom
       AddLine(0,0,0.05,0.2,2,rcEndSmooth);
       //add a straight wall
       AddLine(0,0,0.6,0.2,2);
       //add a transition to a smaller radius
       AddLine(0,0,0.3,0.08,5,rcSmooth);
       //add another straight wall
       AddLine(0,0,0.2,0.08,2);
       //make the open top look rounded
       AddLine(0,0,0.01,0.07,3,rcStartSmooth);
       AddLine(0,0,-0.01,0.06,3,rcEndSmooth);
       //add a bit of the inside wall
       AddLine(0,0,-0.2,0.06,1);
      end;
   3: begin  //Create a scoop
       //start at center of the bottom
       StartSegment.Radius:=0.001;
       //add the round parts
       AddLine(0,0,0.3,0.3,10,rcEndSmooth);
       AddLine(0,0,0.005,0.295,3,rcStartSmooth);
       AddLine(0,0,-0.005,0.29,3,rcEndSmooth);
       AddLine(0,0,-0.29,0.001,10,rcStartSmooth);
       //move (invisible) inside the wall (radius=0)
       AddBend(0,0,0,0,1);
       AddBend(0.3,0,0.27,0,1);
       AddLine(0,0,0,0.03,1);
       //add the handle
       AddLine(0.5,0,0.25,0.03,2);
       AddLine(0.1,0,0.05,0.08,3,rcEndSmooth);
       AddLine(0.5,0,0.25,0.08,2);
       AddLine(0.05,0,0.025,0.001,3,rcStartSmooth);
      end;
   4: begin //Create a Helix
       StartSegment.Radius:=0.001;
       //this will create the helix! (consisting of circular bends)
       for a:=1 to 4 do
        begin
         AddBend(0.2-0.1,0.2,0,0.02);
         AddBend(-0.2,-0.2+0.1,0,0.02);
        end;
       AddBend(0.2-0.1,0.2,0,0.001);
      end;
   5: begin //Create a random bend
       randomize;
       StartSegment.Radius:=0.001;
       for a:=1 to 3 do
        AddBend((random(21)-5)/70,(random(21)-5)/70,(random(41)-5)/70,(random(21)-10)/70+0.05,10,TRadChange(random(4)));
       AddBend(0,0,0,0.001,1);
      end;
  end; //of case
 Cached:=false;
 Cached:=true;
 end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
Index:=1;
InitTube(Index);
end;

procedure TForm1.MenuCupClick(Sender: TObject);
begin
VisTube1.Node:=nil;
//Rebuild the VisTube (new object selected)
while VisNode2.Count<>0 do VisNode2.Items[0].free;
//the node is cleared to free possibly loaded Objects
//for the same reason VisTube1.Node is set again
VisTube1.Node:=VisNode2;
Index:=(sender as TMenuItem).Tag;
(sender as TMenuItem).Checked:=true;
//rebuild Tube
InitTube(Index);
end;

procedure TForm1.VisTimer1Timer(sender: TObject; lagcount: Integer);
begin
//turn Node2
VisNode2.Transformation.Rotation.incZ(4);
VisView1.InvalidateGL;
end;

procedure TForm1.Save1Click(Sender: TObject);
begin
//Save Node2 to file
if SaveDialog1.Execute then
  begin
    VisExchange1.FileName:=SaveDialog1.FileName;
    VisExchange1.SavetoFile(VisNode2);
  end;
end;

procedure TForm1.Load1Click(Sender: TObject);
begin
//Load VisGl-File to Node2
if OpenDialog1.Execute then
  begin
   VisTube1.Node:=nil;
   while VisNode2.Count<>0 do VisNode2.Items[0].free;
   VisExchange1.FileName:=OpenDialog1.FileName;
   VisExchange1.LoadFromFile(VisNode2);
  end;
end;

end.
